<?php

namespace App\Http\Controllers;

use App\Models\Customer;
use App\Models\Design;
use App\Models\Order;
use App\Models\PackagingList;
use App\Models\Product;
use App\Models\PurchaseOrder;
use App\Models\Roll;
use App\Models\Supplier;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DesignController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index() {
        return Design::with('order', 'design_master.design_master_details.product', 'rolls.product',
            'rolls.processing_house')->latest()->get();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request) {
        // Use a database transaction for atomicity
        DB::beginTransaction();

        try {
            // Get latest design
            $latestDesign = Design::query()->latest()->first();
            
            // Generate the production number
            $latestId = $latestDesign ? intval(substr($latestDesign->production_no, 4)) : 0;
            $productionNumber = 'PROD'.str_pad(($latestId + 1), 5, '0', STR_PAD_LEFT);

            // Create the design in memory
            $design = new Design($request->all());

            // Update the design with the production number
            $design["production_no"] = $productionNumber;
            // save the design in the database
            $design->save();

            // Decrease product quantities
            if ($request->has('products')) {
                foreach ($request->products as $detail) {
                    $product = Product::find($detail['product_id']);
                    $product->quantity -= (int) $detail['quantity'];
                    $product->save();
                }
            }

            // Commit the transaction
            DB::commit();

            return $design;
        } catch (\Exception $e) {
            // Something went wrong, rollback the transaction
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Design  $rating
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id) {
        return Design::find($id)->update($request->all());
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Design  $rating
     * @return \Illuminate\Http\Response
     */
    public function show($id) {
        return Design::where('id', $id)->with('order', 'design_master', 'rolls.product',
            'rolls.processing_house')->first();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Design  $rating
     * @return \Illuminate\Http\Response
     */
    public function destroy($id) {
        return Design::find($id)->delete();
    }

    public function count() {
        return response([
            'suppliers' => Supplier::count(),
            'customers' => Customer::count(),
            'designs' => Design::count(),
            'products' => Product::count(),
            'purchase_orders' => PurchaseOrder::count(),
            'packaging_lists' => PackagingList::count(),
            'main_orders' => Order::count(),
            'rolls' => Roll::count(),
        ]);
    }
}
